#ifndef NXCDEFS_H
#define NXCDEFS_H
//==============================================================================
//
// Copyright (C) 2006 - John Hansen. All rights reserved.
//
// Workfile:: NXCDefs.h
// Date:: 2007-08-05
// Revision:: 30
//
//------------------------------------------------------------------------------
//
// Contains declarations for the NXC NXT API resources
//
//==============================================================================

#include "NBCCommon.h"

#define u8 unsigned char
#define s8 char
#define u16 unsigned int
#define s16 int
#define u32 unsigned long
#define s32 long

#define S1 0
#define S2 1
#define S3 2
#define S4 3

#define SENSOR_1 Sensor(S1)
#define SENSOR_2 Sensor(S2)
#define SENSOR_3 Sensor(S3)
#define SENSOR_4 Sensor(S4)

#define SENSOR_TYPE_NONE            IN_TYPE_NO_SENSOR
#define SENSOR_TYPE_TOUCH           IN_TYPE_SWITCH
#define SENSOR_TYPE_TEMPERATURE     IN_TYPE_TEMPERATURE
#define SENSOR_TYPE_LIGHT           IN_TYPE_REFLECTION
#define SENSOR_TYPE_ROTATION        IN_TYPE_ANGLE
#define SENSOR_TYPE_LIGHT_ACTIVE    IN_TYPE_LIGHT_ACTIVE
#define SENSOR_TYPE_LIGHT_INACTIVE  IN_TYPE_LIGHT_INACTIVE
#define SENSOR_TYPE_SOUND_DB        IN_TYPE_SOUND_DB
#define SENSOR_TYPE_SOUND_DBA       IN_TYPE_SOUND_DBA
#define SENSOR_TYPE_CUSTOM          IN_TYPE_CUSTOM
#define SENSOR_TYPE_LOWSPEED        IN_TYPE_LOWSPEED
#define SENSOR_TYPE_LOWSPEED_9V     IN_TYPE_LOWSPEED_9V
#define SENSOR_TYPE_HIGHSPEED       IN_TYPE_HISPEED

#define SENSOR_MODE_RAW		IN_MODE_RAW
#define SENSOR_MODE_BOOL	IN_MODE_BOOLEAN
#define SENSOR_MODE_EDGE	IN_MODE_TRANSITIONCNT
#define SENSOR_MODE_PULSE	IN_MODE_PERIODCOUNTER
#define SENSOR_MODE_PERCENT	IN_MODE_PCTFULLSCALE
#define SENSOR_MODE_CELSIUS	IN_MODE_CELSIUS
#define SENSOR_MODE_FAHRENHEIT	IN_MODE_FAHRENHEIT
#define SENSOR_MODE_ROTATION	IN_MODE_ANGLESTEP

#define _SENSOR_CFG(_type,_mode)	(((_type)<<8)+(_mode))
#define SENSOR_TOUCH		_SENSOR_CFG(SENSOR_TYPE_TOUCH, SENSOR_MODE_BOOL)
#define SENSOR_LIGHT		_SENSOR_CFG(SENSOR_TYPE_LIGHT, SENSOR_MODE_PERCENT)
#define SENSOR_ROTATION		_SENSOR_CFG(SENSOR_TYPE_ROTATION, SENSOR_MODE_ROTATION)
#define SENSOR_CELSIUS		_SENSOR_CFG(SENSOR_TYPE_TEMPERATURE, SENSOR_MODE_CELSIUS)
#define SENSOR_FAHRENHEIT	_SENSOR_CFG(SENSOR_TYPE_TEMPERATURE, SENSOR_MODE_FAHRENHEIT)
#define	SENSOR_PULSE		_SENSOR_CFG(SENSOR_TYPE_TOUCH, SENSOR_MODE_PULSE)
#define SENSOR_EDGE		_SENSOR_CFG(SENSOR_TYPE_TOUCH, SENSOR_MODE_EDGE)

#define SetSensor(_sensor, _tm) { SetSensorType(_sensor, _tm>>8); SetSensorMode(_sensor, _tm&0xff); }

#define PlayTone(_f, _d) PlayToneEx(_f, _d, 4, 0)
#define PlayFile(_f) PlayFileEx(_f, 4, 0)
#define ClearScreen() asm { PointOutEx(200, 200, TRUE) }

// input fields
#define Sensor(p) asm { ReadSensor(p, __RETVAL__) }
#define SensorUS(p) asm { ReadSensorUS(p, __RETVAL__) }
#define SensorType(p) GetInput(p, Type)
#define SensorMode(p) GetInput(p, InputMode)
#define SensorRaw(p) GetInput(p, RawValue)
#define SensorNormalized(p) GetInput(p, NormalizedValue)
#define SensorScaled(p) GetInput(p, ScaledValue)
#define SensorInvalid(p) GetInput(p, InvalidData)

// output fields
#define MotorMode(p) GetOutput(p, OutputMode)
#define MotorPower(p) GetOutput(p, Power)
#define MotorActualSpeed(p) GetOutput(p, ActualSpeed)
#define MotorTachoCount(p) GetOutput(p, TachoCount)
#define MotorTachoLimit(p) GetOutput(p, TachoLimit)
#define MotorRunState(p) GetOutput(p, RunState)
#define MotorTurnRatio(p) GetOutput(p, TurnRatio)
#define MotorRegulation(p) GetOutput(p, RegMode)
#define MotorOverload(p) GetOutput(p, Overload)
#define MotorRegPValue(p) GetOutput(p, RegPValue)
#define MotorRegIValue(p) GetOutput(p, RegIValue)
#define MotorRegDValue(p) GetOutput(p, RegDValue)
#define MotorBlockTachoCount(p) GetOutput(p, BlockTachoCount)
#define MotorRotationCount(p) GetOutput(p, RotationCount)

#define until(c) while(!(c))

#define FreeMemory() asm { GetFreeMemory(__RETVAL__) }
#define BatteryLevel() asm { GetBatteryLevel(__TMPWORD__) __RETURN__ __TMPWORD__ }
#define SoundState() asm { GetSoundState(__RETVAL__, __TMPBYTE__) }
#define SoundFlags() asm { GetSoundState(__TMPBYTE__, __RETVAL__) }
#define StopSound() asm { __setSoundState(SOUND_STATE_STOP, 0, __RETVAL__) }
#define PowerDown() asm { SetIOCtrlModuleValue(IOCtrlOffsetPowerOn, IOCTRL_POWERDOWN) }
#define RebootInFirmwareMode() asm { SetIOCtrlModuleValue(IOCtrlOffsetPowerOn, IOCTRL_BOOT) }
#define CurrentTick() asm { gettick __RETVAL__ }
#define FirstTick() asm { GetFirstTick(__RETVAL__) }
#define ResetSleepTimer() asm { acquire __KeepAliveMutex \
  syscall KeepAlive, __KeepAliveArgs \
  mov __RETVAL__, __KeepAliveArgs.Result \
  release __KeepAliveMutex }

#define IOMA(n) asm { mov __RETVAL__, n }
#define SetIOMA(n, val) asm { mov n, val }

#define ByteArrayToStr(asrc) asm { arrtostr __STRRETVAL__, asrc }
#define ByteArrayToStrEx(asrc, sout) asm { arrtostr sout, asrc }
#define StrToByteArray(ssrc, aout) asm { strtoarr aout, ssrc }
#define ArrayLen(asrc) asm { arrsize __RETVAL__, asrc }
#define ArrayInit(aout, val, cnt) asm { arrinit aout, val, cnt }
#define ArraySubset(aout, asrc, idx, len) asm { arrsubset aout, asrc, idx, len }

#define GetLSInputBuffer(p, offset, cnt, data)  asm { GetIOMapBytes(LowSpeedModuleName, LowSpeedOffsetInBufBuf(p)+offset, cnt, data) }
#define GetLSOutputBuffer(p, offset, cnt, data) asm { GetIOMapBytes(LowSpeedModuleName, LowSpeedOffsetOutBufBuf(p)+offset, cnt, data) }
#define GetDisplayNormal(x, line, cnt, data) asm { GetIOMapBytes(DisplayModuleName, DisplayOffsetNormal(line,x), cnt, data) }
#define GetDisplayPopup(x, line, cnt, data) asm { GetIOMapBytes(DisplayModuleName, DisplayOffsetPopup(line,x), cnt, data) }
#define GetBTInputBuffer(offset, cnt, data) asm { GetIOMapBytes(CommModuleName, CommOffsetBtInBufBuf+offset, cnt, data) }
#define GetBTOutputBuffer(offset, cnt, data) asm { GetIOMapBytes(CommModuleName, CommOffsetBtOutBufBuf+offset, cnt, data) }
#define GetHSInputBuffer(offset, cnt, data) asm { GetIOMapBytes(CommModuleName, CommOffsetHsInBufBuf+offset, cnt, data) }
#define GetHSOutputBuffer(offset, cnt, data) asm { GetIOMapBytes(CommModuleName, CommOffsetHsOutBufBuf+offset, cnt, data) }
#define GetUSBInputBuffer(offset, cnt, data) asm { GetIOMapBytes(CommModuleName, CommOffsetUsbInBufBuf+offset, cnt, data) }
#define GetUSBOutputBuffer(offset, cnt, data) asm { GetIOMapBytes(CommModuleName, CommOffsetUsbOutBufBuf+offset, cnt, data) }
#define GetUSBPollBuffer(offset, cnt, data) asm { GetIOMapBytes(CommModuleName, CommOffsetUsbPollBufBuf+offset, cnt, data) }

#define BTDeviceName(p) asm { GetIOMapBytes(CommModuleName, CommOffsetBtDeviceTableName(p), 16, __STRRETVAL__) }
#define BTConnectionName(p) asm { GetIOMapBytes(CommModuleName, CommOffsetBtConnectTableName(p), 16, __STRRETVAL__) }
#define BTConnectionPinCode(p) asm { GetIOMapBytes(CommModuleName, CommOffsetBtConnectTablePinCode(p), 16, __STRRETVAL__) }
#define BrickDataName() asm { GetIOMapBytes(CommModuleName, CommOffsetBrickDataName, 16, __STRRETVAL__) }

#define GetBTDeviceAddress(p, data) asm { GetIOMapBytes(CommModuleName, CommOffsetBtDeviceTableBdAddr(p), 7, data) }
#define GetBTConnectionAddress(p, data) asm { GetIOMapBytes(CommModuleName, CommOffsetBtConnectTableBdAddr(p), 7, data) }
#define GetBrickDataAddress(data) asm { GetIOMapBytes(CommModuleName, CommOffsetBrickDataBdAddr, 7, data) }

#define SoundFrequency() asm { GetSoundFrequency(__TMPWORD__) __RETURN__ __TMPWORD__ }
#define SoundDuration() asm { GetSoundDuration(__TMPWORD__) __RETURN__ __TMPWORD__ }
#define SoundSampleRate() asm { GetSoundSampleRate(__TMPWORD__) __RETURN__ __TMPWORD__ }
#define SoundMode() asm { GetSoundMode(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define SoundVolume() asm { GetSoundVolume(__TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define ButtonPressCount(b) asm { GetButtonPressCount(b, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define ButtonLongPressCount(b) asm { GetButtonLongPressCount(b, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define ButtonShortReleaseCount(b) asm { GetButtonShortReleaseCount(b, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define ButtonLongReleaseCount(b) asm { GetButtonLongReleaseCount(b, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define ButtonReleaseCount(b) asm { GetButtonReleaseCount(b, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define ButtonState(b) asm { GetButtonState(b, __TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define CommandFlags() asm { GetCommandFlags(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define UIState() asm { GetUIState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define UIButton() asm { GetUIButton(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define VMRunState() asm { GetVMRunState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BatteryState() asm { GetBatteryState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BluetoothState() asm { GetBluetoothState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define UsbState() asm { GetUsbState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define SleepTimeout() asm { GetSleepTimeout(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define SleepTimer() asm { GetSleepTimer(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define RechargeableBattery() asm { GetRechargeableBattery(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define Volume() asm { GetVolume(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define OnBrickProgramPointer() asm { GetOnBrickProgramPointer(__TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define CustomSensorZeroOffset(p) asm { GetInCustomZeroOffset(p, __TMPWORD__) __RETURN__ __TMPWORD__ }
#define CustomSensorPercentFullScale(p) asm { GetInCustomPercentFullScale(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define CustomSensorActiveStatus(p) asm { GetInCustomActiveStatus(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define SensorBoolean(p) asm { GetInSensorBoolean(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define SensorDigiPinsDirection(p) asm { GetInDigiPinsDirection(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define SensorDigiPinsStatus(p) asm { GetInDigiPinsStatus(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define SensorDigiPinsOutputLevel(p) asm { GetInDigiPinsOutputLevel(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define MotorPwnFreq() asm { GetOutPwnFreq(__TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define LSInputBufferInPtr(p) asm { GetLSInputBufferInPtr(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSInputBufferOutPtr(p) asm { GetLSInputBufferOutPtr(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSInputBufferBytesToRx(p) asm { GetLSInputBufferBytesToRx(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSOutputBufferInPtr(p) asm { GetLSOutputBufferInPtr(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSOutputBufferOutPtr(p) asm { GetLSOutputBufferOutPtr(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSOutputBufferBytesToRx(p) asm { GetLSOutputBufferBytesToRx(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSMode(p) asm { GetLSMode(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSChannelState(p) asm { GetLSChannelState(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSErrorType(p) asm { GetLSErrorType(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSState() asm { GetLSState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define LSSpeed() asm { GetLSSpeed(__TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define DisplayEraseMask() asm { GetDisplayEraseMask(__TMPLONG__) __RETURN__ __TMPLONG__ }
#define DisplayUpdateMask() asm { GetDisplayUpdateMask(__TMPLONG__) __RETURN__ __TMPLONG__ }
#define DisplayDisplay() asm { GetDisplayDisplay(__TMPLONG__) __RETURN__ __TMPLONG__ }
#define DisplayFlags() asm { GetDisplayFlags(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define DisplayTextLinesCenterFlags() asm { GetDisplayTextLinesCenterFlags(__TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define BTDeviceClass(p) asm { GetBTDeviceClass(p, __TMPLONG__) __RETURN__ __TMPLONG__ }
#define BTDeviceStatus(p) asm { GetBTDeviceStatus(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTConnectionClass(p) asm { GetBTConnectionClass(p, __TMPLONG__) __RETURN__ __TMPLONG__ }
#define BTConnectionHandleNum(p) asm { GetBTConnectionHandleNum(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTConnectionStreamStatus(p) asm { GetBTConnectionStreamStatus(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTConnectionLinkQuality(p) asm { GetBTConnectionLinkQuality(p, __TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BrickDataBluecoreVersion() asm { GetBrickDataBluecoreVersion(__TMPWORD__) __RETURN__ __TMPWORD__ }
#define BrickDataBtStateStatus() asm { GetBrickDataBtStateStatus(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BrickDataBtHardwareStatus() asm { GetBrickDataBtHardwareStatus(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BrickDataTimeoutValue() asm { GetBrickDataTimeoutValue(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTInputBufferInPtr() asm { GetBTInputBufferInPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTInputBufferOutPtr() asm { GetBTInputBufferOutPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTOutputBufferInPtr() asm { GetBTOutputBufferInPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTOutputBufferOutPtr() asm { GetBTOutputBufferOutPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define HSInputBufferInPtr() asm { GetHSInputBufferInPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define HSInputBufferOutPtr() asm { GetHSInputBufferOutPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define HSOutputBufferInPtr() asm { GetHSOutputBufferInPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define HSOutputBufferOutPtr() asm { GetHSOutputBufferOutPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define USBInputBufferInPtr() asm { GetUSBInputBufferInPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define USBInputBufferOutPtr() asm { GetUSBInputBufferOutPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define USBOutputBufferInPtr() asm { GetUSBOutputBufferInPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define USBOutputBufferOutPtr() asm { GetUSBOutputBufferOutPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define USBPollBufferInPtr() asm { GetUSBPollBufferInPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define USBPollBufferOutPtr() asm { GetUSBPollBufferOutPtr(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTDeviceCount() asm { GetBTDeviceCount(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define BTDeviceNameCount() asm { GetBTDeviceNameCount(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define HSFlags() asm { GetHSFlags(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define HSSpeed() asm { GetHSSpeed(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define HSState() asm { GetHSState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }
#define USBState() asm { GetUSBState(__TMPBYTE__) __RETURN__ __TMPBYTE__ }

#define SetSoundFrequency(n) asm { __setSoundFrequency(n) }
#define SetSoundDuration(n) asm { __setSoundDuration(n) }
#define SetSoundSampleRate(n) asm { __setSoundSampleRate(n) }
#define SetSoundFlags(n) asm { __setSoundFlags(n) }
#define SetSoundState(n) asm { __setSoundState(n) }
#define SetSoundMode(n) asm { __setSoundMode(n) }
#define SetSoundVolume(n) asm { __setSoundVolume(n) }

#define SetButtonPressCount(b, n) asm { __setButtonPressCount(b, n) }
#define SetButtonLongPressCount(b, n) asm { __setButtonLongPressCount(b, n) }
#define SetButtonShortReleaseCount(b, n) asm { __setButtonShortReleaseCount(b, n) }
#define SetButtonLongReleaseCount(b, n) asm { __setButtonLongReleaseCount(b, n) }
#define SetButtonReleaseCount(b, n) asm { __setButtonReleaseCount(b, n) }
#define SetButtonState(b, n) asm { __setButtonState(b, n) }

#define SetCommandFlags(n) asm { __setCommandFlags(n) }
#define SetUIState(n) asm { __setUIState(n) }
#define SetUIButton(n) asm { __setUIButton(n) }
#define SetVMRunState(n) asm { __setVMRunState(n) }
#define SetBatteryState(n) asm { __setBatteryState(n) }
#define SetBluetoothState(n) asm { __setBluetoothState(n) }
#define SetUsbState(n) asm { __setUsbState(n) }
#define SetSleepTimeout(n) asm { __setSleepTimeout(n) }
#define SetSleepTimer(n) asm { __setSleepTimer(n) }
#define SetVolume(n) asm { __setVolume(n) }
#define SetOnBrickProgramPointer(n) asm { __setOnBrickProgramPointer(n) }
#define ForceOff(n) asm { __forceOff(n) }

#define SetCustomSensorZeroOffset(p, n) asm { __setInCustomZeroOffset(p, n) }
#define SetCustomSensorPercentFullScale(p, n) asm { __setInCustomPercentFullScale(p, n) }
#define SetCustomSensorActiveStatus(p, n) asm { __setInCustomActiveStatus(p, n) }
#define SetSensorBoolean(p, n) asm { __setInSensorBoolean(p, n) }
#define SetSensorDigiPinsDirection(p, n) asm { __setInDigiPinsDirection(p, n) }
#define SetSensorDigiPinsStatus(p, n) asm { __setInDigiPinsStatus(p, n) }
#define SetSensorDigiPinsOutputLevel(p, n) asm { __setInDigiPinsOutputLevel(p, n) }

#define SetMotorPwnFreq(n) asm { __setOutPwnFreq(n) }

#define SetLSInputBuffer(p, offset, cnt, data) asm { SetIOMapBytes(LowSpeedModuleName, LowSpeedOffsetInBufBuf(p)+offset, cnt, data) }
#define SetLSInputBufferInPtr(p, n) asm { __setLSInputBufferInPtr(p, n) }
#define SetLSInputBufferOutPtr(p, n) asm { __setLSInputBufferOutPtr(p, n) }
#define SetLSInputBufferBytesToRx(p, n) asm { __setLSInputBufferBytesToRx(p, n) }
#define SetLSOutputBuffer(p, offset, cnt, data) asm { SetIOMapBytes(LowSpeedModuleName, LowSpeedOffsetOutBufBuf(p)+offset, cnt, data) }
#define SetLSOutputBufferInPtr(p, n) asm { __setLSOutputBufferInPtr(p, n) }
#define SetLSOutputBufferOutPtr(p, n) asm { __setLSOutputBufferOutPtr(p, n) }
#define SetLSOutputBufferBytesToRx(p, n) asm { __setLSOutputBufferBytesToRx(p, n) }
#define SetLSMode(p, n) asm { __setLSMode(p, n) }
#define SetLSChannelState(p, n) asm { __setLSChannelState(p, n) }
#define SetLSErrorType(p, n) asm { __setLSErrorType(p, n) }
#define SetLSState(n) asm { __setLSState(n) }
#define SetLSSpeed(n) asm { __setLSSpeed(n) }

#define SetDisplayEraseMask(n) asm { __setDisplayEraseMask(n) }
#define SetDisplayUpdateMask(n) asm { __setDisplayUpdateMask(n) }
#define SetDisplayDisplay(n) asm { __setDisplayDisplay(n) }
#define SetDisplayFlags(n) asm { __setDisplayFlags(n) }
#define SetDisplayTextLinesCenterFlags(n) asm { __setDisplayTextLinesCenterFlags(n) }
#define SetDisplayNormal(x, line, cnt, data) asm { SetIOMapBytes(DisplayModuleName, DisplayOffsetNormal(line,x), cnt, data) }
#define SetDisplayPopup(x, line, cnt, data) asm { SetIOMapBytes(DisplayModuleName, DisplayOffsetPopup(line,x), cnt, data) }

#define SetBTDeviceName(p, str) asm { SetIOMapBytes(CommModuleName, CommOffsetBtDeviceTableName(p), 16, str) }
#define SetBTDeviceAddress(p, addr) asm { SetIOMapBytes(CommModuleName, CommOffsetBtDeviceTableBdAddr(p), 7, addr) }
#define SetBTConnectionName(p, str) asm { SetIOMapBytes(CommModuleName, CommOffsetBtConnectTableName(p), 16, str) }
#define SetBTConnectionPinCode(p, code) asm { SetIOMapBytes(CommModuleName, CommOffsetBtConnectTablePinCode(p), 16, code) }
#define SetBTConnectionAddress(p, addr) asm { SetIOMapBytes(CommModuleName, CommOffsetBtConnectTableBdAddr(p), 7, addr) }
#define SetBrickDataName(str) asm { SetIOMapBytes(CommModuleName, CommOffsetBrickDataName, 16, str) }
#define SetBrickDataAddress(p, addr) asm { SetIOMapBytes(CommModuleName, CommOffsetBrickDataBdAddr, 7, addr) }
#define SetBTDeviceClass(p, n) asm { __setBTDeviceClass(p, n) }
#define SetBTDeviceStatus(p, n) asm { __setBTDeviceStatus(p, n) }
#define SetBTConnectionClass(p, n) asm { __setBTConnectionClass(p, n) }
#define SetBTConnectionHandleNum(p, n) asm { __setBTConnectionHandleNum(p, n) }
#define SetBTConnectionStreamStatus(p, n) asm { __setBTConnectionStreamStatus(p, n) }
#define SetBTConnectionLinkQuality(p, n) asm { __setBTConnectionLinkQuality(p, n) }
#define SetBrickDataBluecoreVersion(n) asm { __setBrickDataBluecoreVersion(n) }
#define SetBrickDataBtStateStatus(n) asm { __setBrickDataBtStateStatus(n) }
#define SetBrickDataBtHardwareStatus(n) asm { __setBrickDataBtHardwareStatus(n) }
#define SetBrickDataTimeoutValue(n) asm { __setBrickDataTimeoutValue(n) }
#define SetBTInputBuffer(offset, cnt, data) asm { SetIOMapBytes(CommModuleName, CommOffsetBtInBufBuf+offset, cnt, data) }
#define SetBTInputBufferInPtr(n) asm { __setBTInputBufferInPtr(n) }
#define SetBTInputBufferOutPtr(n) asm { __setBTInputBufferOutPtr(n) }
#define SetBTOutputBuffer(offset, cnt, data) asm { SetIOMapBytes(CommModuleName, CommOffsetBtOutBufBuf+offset, cnt, data) }
#define SetBTOutputBufferInPtr(n) asm { __setBTOutputBufferInPtr(n) }
#define SetBTOutputBufferOutPtr(n) asm { __setBTOutputBufferOutPtr(n) }
#define SetHSInputBuffer(offset, cnt, data) asm { SetIOMapBytes(CommModuleName, CommOffsetHsInBufBuf+offset, cnt, data) }
#define SetHSInputBufferInPtr(n) asm { __setHSInputBufferInPtr(n) }
#define SetHSInputBufferOutPtr(n) asm { __setHSInputBufferOutPtr(n) }
#define SetHSOutputBuffer(offset, cnt, data) asm { SetIOMapBytes(CommModuleName, CommOffsetHsOutBufBuf+offset, cnt, data) }
#define SetHSOutputBufferInPtr(n) asm { __setHSOutputBufferInPtr(n) }
#define SetHSOutputBufferOutPtr(n) asm { __setHSOutputBufferOutPtr(n) }
#define SetUSBInputBuffer(offset, cnt, data) asm { SetIOMapBytes(CommModuleName, CommOffsetUsbInBufBuf+offset, cnt, data) }
#define SetUSBInputBufferInPtr(n) asm { __setUSBInputBufferInPtr(n) }
#define SetUSBInputBufferOutPtr(n) asm { __setUSBInputBufferOutPtr(n) }
#define SetUSBOutputBuffer(offset, cnt, data) asm { SetIOMapBytes(CommModuleName, CommOffsetUsbOutBufBuf+offset, cnt, data) }
#define SetUSBOutputBufferInPtr(n) asm { __setUSBOutputBufferInPtr(n) }
#define SetUSBOutputBufferOutPtr(n) asm { __setUSBOutputBufferOutPtr(n) }
#define SetUSBPollBuffer(offset, cnt, data) asm { SetIOMapBytes(CommModuleName, CommOffsetUsbPollBufBuf+offset, cnt, data) }
#define SetUSBPollBufferInPtr(n) asm { __setUSBPollBufferInPtr(n) }
#define SetUSBPollBufferOutPtr(n) asm { __setUSBPollBufferOutPtr(n) }
#define SetBTDeviceCount(n) asm { __setBTDeviceCount(n) }
#define SetBTDeviceNameCount(n) asm { __setBTDeviceNameCount(n) } 
#define SetHSFlags(n) asm { __setHSFlags(n) }
#define SetHSSpeed(n) asm { __setHSSpeed(n) }
#define SetHSState(n) asm { __setHSState(n) }
#define SetUSBState(n) asm { __setUSBState(n) }


#define CreateFile(_fname, _fsize, _handle) asm { __createFile(_fname, _fsize, _handle, __RETVAL__) }
#define OpenFileAppend(_fname, _fsize, _handle) asm { __openFileAppend(_fname, _fsize, _handle, __RETVAL__) }
#define OpenFileRead(_fname, _fsize, _handle) asm { __openFileRead(_fname, _fsize, _handle, __RETVAL__) }
#define CloseFile(_handle) asm { __closeFile(_handle, __RETVAL__) }
#define ResolveHandle(_fname, _handle, _writeable) asm { __resolveHandle(_fname, _handle, _writeable, __RETVAL__) }
#define RenameFile(_oldname, _newname) asm { __renameFile(_oldname, _newname, __RETVAL__) }
#define DeleteFile(_fname) asm { __deleteFile(_fname, __RETVAL__) }

#define Read(_handle, _n) asm { __readValue(_handle, _n, __RETVAL__) }
#define ReadLn(_handle, _n) asm { __readLnValue(_handle, _n, __RETVAL__) }
#define ReadBytes(_handle, _len, _buf) asm { __readBytes(_handle, _len, _buf, __RETVAL__) }
#define ReadLnString(_handle, _output) asm { __readLnString(_handle, _output, __RETVAL__) }

#define Write(_handle, _n) asm { __writeValue(_handle, _n, __RETVAL__) }
#define WriteLn(_handle, _n) asm { __writeLnValue(_handle, _n, __RETVAL__) }
#define WriteString(_handle, _str, _cnt) asm { __writeString(_handle, _str, _cnt, __RETVAL__) }
#define WriteLnString(_handle, _str, _cnt) asm { __writeLnString(_handle, _str, _cnt, __RETVAL__) }
#define WriteBytes(_handle, _buf, _cnt) asm { __writeBytes(_handle, _buf, _cnt, __RETVAL__) }
#define WriteBytesEx(_handle, _len, _buf) asm { __writeBytesEx(_handle, _len, _buf, __RETVAL__) }

#define SendMessage(_queue, _msg) asm { __sendMessage(_queue, _msg, __RETVAL__) }
#define ReceiveMessage(_queue, _clear, _msg) asm { __receiveMessage(_queue, _clear, _msg, __RETVAL__) }

#define LowspeedStatus(_port, _bready) asm { __lowspeedStatus(_port, _bready, __RETVAL__) }
#define LowspeedCheckStatus(_port) asm { __lowspeedStatus(_port, __TMPBYTE__, __RETVAL__) }
#define LowspeedBytesReady(_port) asm { __lowspeedStatus(_port, __RETVAL__, __TMPBYTE__) }
#define LowspeedWrite(_port, _retlen, _buffer) asm { __lowspeedWrite(_port, _retlen, _buffer, __RETVAL__) }
#define LowspeedRead(_port, _buflen, _buffer) asm { __lowspeedRead(_port, _buflen, _buffer, __RETVAL__) }

#define I2CStatus(_port, _bready) LowspeedStatus(_port, _bready)
#define I2CCheckStatus(_port) LowspeedCheckStatus(_port)
#define I2CBytesReady(_port) LowspeedBytesReady(_port)
#define I2CWrite(_port, _retlen, _buffer) LowspeedWrite(_port, _retlen, _buffer)
#define I2CRead(_port, _buflen, _buffer) LowspeedRead(_port, _buflen, _buffer)

#define I2CBytes(_port, _inbuf, _count, _outbuf) asm { ReadI2CBytes(_port, _inbuf, _count, _outbuf, __RETVAL__) }

#define BluetoothStatus(_conn) asm { __bluetoothStatus(_conn, __RETVAL__) }
#define BluetoothWrite(_conn, _buffer) asm { __bluetoothWrite(_conn, _buffer, __RETVAL__) }

#define SendRemoteBool(_conn, _queue, _bval) asm { __sendRemoteBool(_conn, _queue, _bval, __RETVAL__) }
#define SendRemoteNumber(_conn, _queue, _val) asm { __sendRemoteNumber(_conn, _queue, _val, __RETVAL__) }
#define SendRemoteString(_conn, _queue, _str) asm { __sendRemoteString(_conn, _queue, _str, __RETVAL__) }

#define SendResponseBool(_queue, _bval) asm { __sendResponseBool(_queue, _bval, __RETVAL__) }
#define SendResponseNumber(_queue, _val) asm { __sendResponseNumber(_queue, _val, __RETVAL__) }
#define SendResponseString(_queue, _msg) asm { __sendResponseString(_queue, _msg, __RETVAL__) }

#define ReceiveRemoteBool(_queue, _clear, _bval) asm { __receiveRemoteBool(_queue, _clear, _bval, __RETVAL__) }
#define ReceiveRemoteNumber(_queue, _clear, _val) asm { __receiveRemoteNumber(_queue, _clear, _val, __RETVAL__) }
#define ReceiveRemoteString(_queue, _clear, _str) asm { __receiveMessage(_queue, _clear, _str, __RETVAL__) }
#define ReceiveRemoteMessageEx(_queue, _clear, _str, _val, _bval) asm { __receiveRemoteMessageEx(_queue, _clear, _str, _val, _bval, __RETVAL__) }

#define RemoteMessageRead(_conn, _queue) asm { __remoteMessageRead(_conn, _queue, __RETVAL__) }
#define RemoteMessageWrite(_conn, _queue, _msg) asm { __sendRemoteString(_conn, _queue, _msg, __RETVAL__) }
#define RemoteStartProgram(_conn, _filename) asm { __remoteStartProgram(_conn, _filename, __RETVAL__) }
#define RemoteStopProgram(_conn) asm { __bluetoothWrite(_conn, __DCStopProgramPacket, __RETVAL__) }
#define RemotePlaySoundFile(_conn, _filename, _bloop) asm { __remotePlaySoundFile(_conn, _filename, _bloop, __RETVAL__) }
#define RemotePlayTone(_conn, _frequency, _duration) asm { __remotePlayTone(_conn, _frequency, _duration, __RETVAL__) }
#define RemoteStopSound(_conn) asm { __bluetoothWrite(_conn, __DCStopSoundPacket, __RETVAL__) }
#define RemoteKeepAlive(_conn) asm { __bluetoothWrite(_conn, __DCKeepAlivePacket, __RETVAL__) }
#define RemoteResetScaledValue(_conn, _port) asm { __remoteResetScaledValue(_conn, _port, __RETVAL__) }
#define RemoteResetMotorPosition(_conn, _port, _brelative) asm { __remoteResetMotorPosition(_conn, _port, _brelative, __RETVAL__) }
#define RemoteSetInputMode(_conn, _port, _type, _mode) asm { __remoteSetInputMode(_conn, _port, _type, _mode, __RETVAL__) }
#define RemoteSetOutputState(_conn, _port, _speed, _mode, _regmode, _turnpct, _runstate, _tacholimit) \
  asm { __remoteSetOutputState(_conn, _port, _speed, _mode, _regmode, _turnpct, _runstate, _tacholimit, __RETVAL__) }

// math functions written by Tamas Sorosy (www.sorosy.com)

// X is any integer; Y is the sqrt value (0->max); if X<0, Y is the sqrt value of absolute X
#define Sqrt(X) asm { __SQRT(X,__RETVAL__) }

// X is any integer in degrees; Y is 100* the sin value (-100->100)
#define Sin(X) asm { __SIN(X,__RETVAL__) }

// X is any integer in degrees; Y is 100* the cos value (-100->100)
#define Cos(X) asm { __COS(X,__RETVAL__) }

// X is 100* the sin value (-100->100); Y is -90->90; Y is 101 if X is outside -100->100 range
#define Asin(X) asm { __ASIN(X,__RETVAL__) }

// X is 100* the cos value (-100->100); Y is 0->180; Y is -11 if X is outside -100->100 range
#define Acos(X) asm { __ACOS(X,__RETVAL__) }

// standard syscall structures

struct LocationType {
  int X;
  int Y;
};

struct SizeType {
  int Width;
  int Height;
};

// FileOpenRead, FileOpenWrite, FileOpenAppend
struct FileOpenType {
  unsigned int Result;
  byte FileHandle;
  string Filename;
  unsigned long Length;
};

// FileRead, FileWrite
struct FileReadWriteType {
  unsigned int Result;
  byte FileHandle;
  string Buffer;
  unsigned long Length;
};

// FileClose
struct FileCloseType {
  unsigned int Result;
  byte FileHandle;
};

// FileResolveHandle
struct FileResolveHandleType {
  unsigned int Result;
  byte FileHandle;
  bool WriteHandle;
  string Filename;
};

// FileRename
struct FileRenameType {
  unsigned int Result;
  string OldFilename;
  string NewFilename;
};

// FileDelete
struct FileDeleteType {
  unsigned int Result;
  string Filename;
};

// SoundPlayFile
struct SoundPlayFileType {
  char Result;
  string Filename;
  bool Loop;
  byte SoundLevel;
};

// SoundPlayTone
struct SoundPlayToneType {
  char Result;
  unsigned int Frequency;
  unsigned int Duration;
  bool Loop;
  byte SoundLevel;
};

// SoundGetState
struct SoundGetStateType {
  byte State;
  byte Flags;
};

// SoundSetState
struct SoundSetStateType {
  byte Result;
  byte State;
  byte Flags;
};

// DrawText
struct DrawTextType {
  char Result;
  LocationType Location;
  string Text;
  unsigned long Options;
};

// DrawPoint
struct DrawPointType {
  char Result;
  LocationType Location;
  unsigned long Options;
};

// DrawLine
struct DrawLineType {
  char Result;
  LocationType StartLoc;
  LocationType EndLoc;
  unsigned long Options;
};

// DrawCircle
struct DrawCircleType {
  char Result;
  LocationType Center;
  byte Size;
  unsigned long Options;
};

// DrawRect
struct DrawRectType {
  char Result;
  LocationType Location;
  SizeType Size;
  unsigned long Options;
};

// DrawGraphic
struct DrawGraphicType {
  char Result;
  LocationType Location;
  string Filename;
  int Variables[];
  unsigned long Options;
};

// SetScreenMode
struct SetScreenModeType {
  char Result;
  unsigned long ScreenMode;
};

// ReadButton
struct ReadButtonType {
  char Result;
  byte Index;
  bool Pressed;
  byte Count;
  bool Reset; // reset count after reading?
};

// CommLSWrite
struct CommLSWriteType {
  char Result;
  byte Port;
  byte Buffer[];
  byte ReturnLen;
};

// CommLSRead
struct CommLSReadType {
  char Result;
  byte Port;
  byte Buffer[];
  byte BufferLen;
};

// CommLSCheckStatus
struct CommLSCheckStatusType {
  char Result;
  byte Port;
  byte BytesReady;
};

// RandomNumber
struct RandomNumberType {
  int Result;
};

// GetStartTick
struct GetStartTickType {
  unsigned long Result;
};

// MessageWrite
struct MessageWriteType {
  char Result;
  byte QueueID;
  string Message;
};

// MessageRead
struct MessageReadType {
  char Result;
  byte QueueID;
  bool Remove;
  string Message;
};

// CommBTCheckStatus
struct CommBTCheckStatusType {
  char Result;
  byte Connection;
};

// CommBTWrite
struct CommBTWriteType {
  char Result;
  byte Connection;
  byte Buffer[];
};

// KeepAlive
struct KeepAliveType {
  unsigned long Result;
};

// IOMapRead
struct IOMapReadType {
  char Result;
  string ModuleName;
  unsigned int Offset;
  unsigned int Count;
  byte Buffer[];
};

// IOMapWrite
struct IOMapWriteType {
  char Result;
  string ModuleName;
  unsigned int Offset;
  byte Buffer[];
};

// IOMapReadByID
struct IOMapReadByIDType {
  char Result;
  unsigned long ModuleID;
  unsigned int Offset;
  unsigned int Count;
  byte Buffer[];
};

// IOMapWriteByID
struct IOMapWriteByIDType {
  char Result;
  unsigned long ModuleID;
  unsigned int Offset;
  byte Buffer[];
};

// DisplayExecuteFunction
struct DisplayExecuteFunctionType {
  byte Status;
  byte Cmd;
  byte On;
  byte X1;
  byte Y1;
  byte X2;
  byte Y2;
};

// CommExecuteFunction
struct CommExecuteFunctionType {
  unsigned int Result;
  byte Cmd;
  byte Param1;
  byte Param2;
  byte Param3;
  string Name;
  unsigned int RetVal;
};

// LoaderExecuteFunction
struct LoaderExecuteFunctionType {
  unsigned int Result;
  byte Cmd;
  string Filename;
  byte Buffer[];
  unsigned long Length;
};

#define SysCall(_func, _args) asm { syscall _func, _args }

#define SysFileOpenRead(_args) asm { \
  compchktype _args, FileOpenType \
  syscall FileOpenRead, _args \
}
#define SysFileOpenWrite(_args) asm { \
  compchktype _args, FileOpenType \
  syscall FileOpenWrite, _args \
}
#define SysFileOpenAppend(_args) asm { \
  compchktype _args, FileOpenType \
  syscall FileOpenAppend, _args \
}
#define SysFileRead(_args) asm { \
  compchktype _args, FileReadWriteType \
  syscall FileRead, _args \
}
#define SysFileWrite(_args) asm { \
  compchktype _args, FileReadWriteType \
  syscall FileWrite, _args \
}
#define SysFileClose(_args) asm { \
  compchktype _args, FileCloseType \
  syscall FileClose, _args \
}
#define SysFileResolveHandle(_args) asm { \
  compchktype _args, FileResolveHandleType \
  syscall FileResolveHandle, _args \
}
#define SysFileRename(_args) asm { \
  compchktype _args, FileRenameType \
  syscall FileRename, _args \
}
#define SysFileDelete(_args) asm { \
  compchktype _args, FileDeleteType \
  syscall FileDelete, _args \
}

#define SysSoundPlayFile(_args) asm { \
  compchktype _args, SoundPlayFileType \
  syscall SoundPlayFile, _args \
}
#define SysSoundPlayTone(_args) asm { \
  compchktype _args, SoundPlayToneType \
  syscall SoundPlayTone, _args \
}
#define SysSoundGetState(_args) asm { \
  compchktype _args, SoundGetStateType \
  syscall SoundGetState, _args \
}
#define SysSoundSetState(_args) asm { \
  compchktype _args, SoundSetStateType \
  syscall SoundSetState, _args \
}

#define SysDrawText(_args) asm { \
  compchktype _args, DrawTextType \
  syscall DrawText, _args \
}
#define SysDrawPoint(_args) asm { \
  compchktype _args, DrawPointType \
  syscall DrawPoint, _args \
}
#define SysDrawLine(_args) asm { \
  compchktype _args, DrawLineType \
  syscall DrawLine, _args \
}
#define SysDrawCircle(_args) asm { \
  compchktype _args, DrawCircleType \
  syscall DrawCircle, _args \
}
#define SysDrawRect(_args) asm { \
  compchktype _args, DrawRectType \
  syscall DrawRect, _args \
}
#define SysDrawGraphic(_args) asm { \
  compchktype _args, DrawGraphicType \
  syscall DrawGraphic, _args \
}
#define SysSetScreenMode(_args) asm { \
  compchktype _args, SetScreenModeType \
  syscall SetScreenMode, _args \
}

#define SysReadButton(_args) asm { \
  compchktype _args, ReadButtonType \
  syscall ReadButton, _args \
}

#define SysCommLSWrite(_args) asm { \
  compchktype _args, CommLSWriteType \
  syscall CommLSWrite, _args \
}
#define SysCommLSRead(_args) asm { \
  compchktype _args, CommLSReadType \
  syscall CommLSRead, _args \
}
#define SysCommLSCheckStatus(_args) asm { \
  compchktype _args, CommLSCheckStatusType \
  syscall CommLSCheckStatus, _args \
}

#define SysRandomNumber(_args) asm { \
  compchktype _args, RandomNumberType \
  syscall RandomNumber, _args \
}

#define SysGetStartTick(_args) asm { \
  compchktype _args, GetStartTickType \
  syscall GetStartTick, _args \
}

#define SysMessageWrite(_args) asm { \
  compchktype _args, MessageWriteType \
  syscall MessageWrite, _args \
}
#define SysMessageRead(_args) asm { \
  compchktype _args, MessageReadType \
  syscall MessageRead, _args \
}

#define SysCommBTWrite(_args) asm { \
  compchktype _args, CommBTWriteType \
  syscall CommBTWrite, _args \
}
#define SysCommBTCheckStatus(_args) asm { \
  compchktype _args, CommBTCheckStatusType \
  syscall CommBTCheckStatus, _args \
}

#define SysKeepAlive(_args) asm { \
  compchktype _args, KeepAliveType \
  syscall KeepAlive, _args \
}

#define SysIOMapRead(_args) asm { \
  compchktype _args, IOMapReadType \
  syscall IOMapRead, _args \
}
#define SysIOMapWrite(_args) asm { \
  compchktype _args, IOMapWriteType \
  syscall IOMapWrite, _args \
}
#define SysIOMapReadByID(_args) asm { \
  compchktype _args, IOMapReadByIDType \
  syscall IOMapReadByID, _args \
}
#define SysIOMapWriteByID(_args) asm { \
  compchktype _args, IOMapWriteByIDType \
  syscall IOMapWriteByID, _args \
}

#define SysDisplayExecuteFunction(_args) asm { \
  compchktype _args, DisplayExecuteFunctionType \
  syscall DisplayExecuteFunction, _args \
}
#define SysCommExecuteFunction(_args) asm { \
  compchktype _args, CommExecuteFunctionType \
  syscall CommExecuteFunction, _args \
}
#define SysLoaderExecuteFunction(_args) asm { \
  compchktype _args, LoaderExecuteFunctionType \
  syscall LoaderExecuteFunction, _args \
}

#endif // NXCDEFS_H
